@@ -38,8 +38,8 @@ static int _verbose = 0;
  * Library users can provide their own logging
  * function.
  */
-static void _default_log(int level, const char *file, int line,
-			 const char *f, ...)
+static void _default_log(int level, const char *file __attribute((unused)),
+			 int line __attribute((unused)), const char *f, ...)
 {
 	va_list ap;
 
@@ -101,8 +101,10 @@ struct dm_task *dm_task_create(int type)
 		return NULL;
 	}
 
-	if (!dm_check_version())
+	if (!dm_check_version()) {
+		dm_free(dmt);
 		return NULL;
+	}
 
 	memset(dmt, 0, sizeof(*dmt));
 
@@ -510,3 +512,4 @@ out:
 	dm_task_destroy(dmt);
 	return r;
 }
+
